import { useEffect, useState, useCallback } from 'react';
import { isClient } from './util';
var noop = function () { };
var isUndefined = function (value) { return typeof value === 'undefined'; };
var useLocalStorage = function (key, initialValue, raw) {
    if (!isClient) {
        return [initialValue, noop, noop];
    }
    var _a = useState(function () {
        try {
            var localStorageValue = localStorage.getItem(key);
            if (isUndefined(initialValue)) {
                return undefined;
            }
            if (typeof localStorageValue !== 'string') {
                localStorage.setItem(key, raw ? String(initialValue) : JSON.stringify(initialValue));
                return initialValue;
            }
            return raw ? localStorageValue : JSON.parse(localStorageValue || 'null');
        }
        catch (_a) {
            // If user is in private mode or has storage restriction
            // localStorage can throw. JSON.parse and JSON.stringify
            // can throw, too.
            return initialValue;
        }
    }), state = _a[0], setState = _a[1];
    var remove = useCallback(function () {
        try {
            localStorage.removeItem(key);
            setState(undefined);
        }
        catch (_a) {
            // If user is in private mode or has storage restriction
            // localStorage can throw.
        }
    }, [key, setState]);
    useEffect(function () {
        if (isUndefined(state))
            return;
        try {
            var serializedState = raw ? String(state) : JSON.stringify(state);
            localStorage.setItem(key, serializedState);
        }
        catch (_a) {
            // If user is in private mode or has storage restriction
            // localStorage can throw. Also JSON.stringify can throw.
        }
    }, [state]);
    return [state, setState, remove];
};
export default useLocalStorage;
